@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 #include "mutt.h"
@@ -26,13 +26,17 @@
 #include "mx.h"
 #include "pager.h"
 
+#ifdef USE_IMAP
+#include "imap.h"
+#endif
+
 #ifdef BUFFY_SIZE
 #include "buffy.h"
 #endif
 
 
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
 #include "pgp.h"
 #endif
 
@@ -64,7 +68,7 @@ int mutt_display_message (HEADER *cur)
 {
   char tempfile[_POSIX_PATH_MAX], buf[LONG_STRING];
   int rc = 0, builtin = 0;
-  int cmflags = M_CM_DECODE | M_CM_DISPLAY;
+  int cmflags = M_CM_DECODE | M_CM_DISPLAY | M_CM_CHARCONV;
   FILE *fpout;
 
   snprintf (buf, sizeof (buf), "%s/%s", TYPE (cur->content),
@@ -73,8 +77,7 @@ int mutt_display_message (HEADER *cur)
   mutt_parse_mime_message (Context, cur);
 
 
-
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
   /* see if PGP is needed for this message.  if so, we should exit curses */
   if (cur->pgp)
   {
@@ -84,7 +87,6 @@ int mutt_display_message (HEADER *cur)
 	return 0;
 
       cmflags |= M_CM_VERIFY;
-      mutt_message _("Invoking PGP...");
     }
     else if (cur->pgp & PGPSIGN)
     {
@@ -92,18 +94,20 @@ int mutt_display_message (HEADER *cur)
       if (query_quadoption (OPT_VERIFYSIG, _("Verify PGP signature?")) == M_YES)
       {
 	cmflags |= M_CM_VERIFY;
-	mutt_message _("Invoking PGP...");
       }
     }
   }
+  
+  if ((cmflags & M_CM_VERIFY) || (cur->pgp & PGPENCRYPT))
+  {
+    if (cur->env->from)
+      pgp_invoke_getkeys (cur->env->from);
+
+    mutt_message _("Invoking PGP...");
+  }
+
 #endif
 
-
-
-
-
-
-
   mutt_mktemp (tempfile);
   if ((fpout = safe_fopen (tempfile, "w")) == NULL)
   {
@@ -135,10 +139,20 @@ int mutt_display_message (HEADER *cur)
     return (0);
   }
 
+#ifdef HAVE_PGP
+  /* update PGP information for this message */
+  cur->pgp |= pgp_query (cur->content);
+#endif
+
   if (builtin)
   {
     pager_t info;
-    
+
+#ifdef HAVE_PGP
+    if (cur->pgp & PGPGOODSIGN)
+      mutt_message _("PGP signature successfully verified.");
+#endif
+
     /* Invoke the builtin pager */
     memset (&info, 0, sizeof (pager_t));
     info.hdr = cur;
@@ -147,13 +161,17 @@ int mutt_display_message (HEADER *cur)
   }
   else
   {
+    int r;
+
     endwin ();
     snprintf (buf, sizeof (buf), "%s %s", NONULL(Pager), tempfile);
-    mutt_system (buf);
+    if ((r = mutt_system (buf)) == -1)
+      mutt_error (_("Error running \"%s\"!"), buf);
     unlink (tempfile);
     keypad (stdscr, TRUE);
-    mutt_set_flag (Context, cur, M_READ, 1);
-    if (option (OPTPROMPTAFTER))
+    if (r != -1)
+      mutt_set_flag (Context, cur, M_READ, 1);
+    if (r != -1 && option (OPTPROMPTAFTER))
     {
       mutt_ungetch (mutt_any_key_to_continue _("Command: "), 0);
       rc = km_dokey (MENU_PAGER);
@@ -209,18 +227,38 @@ void ci_bounce_message (HEADER *h, int *redraw)
     return;
   }
 
-  mutt_bounce_message (h, adr);
+  mutt_bounce_message (NULL, h, adr);
   rfc822_free_address (&adr);
   mutt_message (h ? _("Message bounced.") : _("Messages bounced."));
 }
 
+static void pipe_print_set_flags (int decode, int *cmflags, int *chflags)
+{
+  if (decode)
+  {
+    *cmflags |= M_CM_DECODE | M_CM_CHARCONV;
+    *chflags |= CH_DECODE | CH_REORDER;
+    
+    if (option (OPTWEED))
+    {
+      *chflags |= CH_WEED;
+      *cmflags |= M_CM_WEED;
+    }
+  }
+}
+
 void mutt_pipe_message_to_state (HEADER *h, STATE *s)
 {
+  int cmflags = 0;
+  int chflags = CH_FROM;
+  
+  pipe_print_set_flags (option (OPTPIPEDECODE), &cmflags, &chflags);
+  
   if (option (OPTPIPEDECODE))
     mutt_parse_mime_message (Context, h);
+
   mutt_copy_message (s->fpout, Context, h,
-		     option (OPTPIPEDECODE) ? M_CM_DECODE : 0,
-		     option (OPTPIPEDECODE) ? CH_FROM | CH_WEED | CH_DECODE | CH_REORDER : CH_FROM);
+		     cmflags, chflags);
 }
 
 int mutt_pipe_message (HEADER *h)
@@ -244,7 +282,7 @@ int mutt_pipe_message (HEADER *h)
 
 
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
     if (option (OPTPIPEDECODE))
     {
       mutt_parse_mime_message (Context, h);
@@ -266,7 +304,7 @@ int mutt_pipe_message (HEADER *h)
 
 
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
 
     if(option(OPTPIPEDECODE))
     {
@@ -433,62 +471,79 @@ void mutt_enter_command (void)
     set_option (OPTNEEDRESORT);
 }
 
-void mutt_display_address (ADDRESS *adr)
+void mutt_display_address (ENVELOPE *env)
 {
+  char *pfx = NULL;
   char buf[SHORT_STRING];
+  ADDRESS *adr = NULL;
+
+  adr = mutt_get_address (env, &pfx);
+
+  if (!adr) return;
 
   buf[0] = 0;
   rfc822_write_address (buf, sizeof (buf), adr);
-  mutt_message ("%s", buf);
+  mutt_message ("%s: %s", pfx, buf);
 }
 
-static void set_copy_flags(HEADER *hdr, int decode, int decrypt, int *cmflags, int *chflags)
+static void set_copy_flags (HEADER *hdr, int decode, int decrypt, int *cmflags, int *chflags)
 {
   *cmflags = 0;
   *chflags = CH_UPDATE_LEN;
   
-#ifdef _PGPPATH
-  if(!decode && decrypt && (hdr->pgp & PGPENCRYPT))
+#ifdef HAVE_PGP
+  if (!decode && decrypt && (hdr->pgp & PGPENCRYPT))
   {
-    if(mutt_is_multipart_encrypted(hdr->content))
+    if (mutt_is_multipart_encrypted(hdr->content))
     {
       *chflags = CH_NONEWLINE | CH_XMIT | CH_MIME;
       *cmflags = M_CM_DECODE_PGP;
     }
-    else if(mutt_is_application_pgp(hdr->content) & PGPENCRYPT)
+    else if (mutt_is_application_pgp(hdr->content) & PGPENCRYPT)
       decode = 1;
   }
 #endif
 
-  if(decode)
+  if (decode)
   {
     *chflags = CH_XMIT | CH_MIME | CH_TXTPLAIN;
-    *cmflags = M_CM_DECODE;
+    *cmflags = M_CM_DECODE | M_CM_CHARCONV;
   }
 
+  /* respect $weed only if decode doesn't kick in
+   * for decrypt.
+   */
+
+  if (decode && !decrypt && option (OPTWEED))
+  {
+    *chflags |= CH_WEED;
+    *cmflags |= M_CM_WEED;
+  }
 }
 
 static void _mutt_save_message (HEADER *h, CONTEXT *ctx, int delete, int decode, int decrypt)
 {
   int cmflags, chflags;
   
-  set_copy_flags(h, decode, decrypt, &cmflags, &chflags);
+  set_copy_flags (h, decode, decrypt, &cmflags, &chflags);
 
-  if (decode  || decrypt)
+  if (decode || decrypt)
     mutt_parse_mime_message (Context, h);
 
   if (mutt_append_message (ctx, Context, h, cmflags, chflags) == 0 && delete)
   {
     mutt_set_flag (Context, h, M_DELETE, 1);
-    mutt_set_flag (Context, h, M_TAG, 0);
+    if (option (OPTDELETEUNTAG))
+      mutt_set_flag (Context, h, M_TAG, 0);
   }
 }
 
 /* returns 0 if the copy/save was successful, or -1 on error/abort */
-int mutt_save_message (HEADER *h, int delete, int decode, int decrypt, int *redraw)
+int mutt_save_message (HEADER *h, int delete, 
+		       int decode, int decrypt, int *redraw)
 {
   int i, need_buffy_cleanup;
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
   int need_passphrase = 0;
 #endif
   char prompt[SHORT_STRING], buf[_POSIX_PATH_MAX];
@@ -509,7 +564,7 @@ int mutt_save_message (HEADER *h, int delete, int decode, int decrypt, int *redr
   
   if (h)
   {
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
     need_passphrase = h->pgp & PGPENCRYPT;
 #endif
     mutt_default_save (buf, sizeof (buf), h);
@@ -530,7 +585,7 @@ int mutt_save_message (HEADER *h, int delete, int decode, int decrypt, int *redr
     if (h)
     {
       mutt_default_save (buf, sizeof (buf), h);
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
       need_passphrase |= h->pgp & PGPENCRYPT;
 #endif
       h = NULL;
@@ -569,13 +624,29 @@ int mutt_save_message (HEADER *h, int delete, int decode, int decrypt, int *redr
     return (-1);
   }
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
   if(need_passphrase && (decode || decrypt) && !pgp_valid_passphrase())
     return -1;
 #endif
   
   mutt_message (_("Copying to %s..."), buf);
   
+#ifdef USE_IMAP
+  if (Context->magic == M_IMAP && 
+      !(decode || decrypt) && mx_is_imap (buf))
+  {
+    switch (imap_copy_messages (Context, h, buf, delete))
+    {
+      /* success */
+      case 0: mutt_clear_error (); return 0;
+      /* non-fatal error: fall through to fetch/append */
+      case 1: break;
+      /* fatal error, abort */
+      case -1: return -1;
+    }
+  }
+#endif
+
   if (mx_open_mailbox (buf, M_APPEND, &ctx) != NULL)
   {
     if (h)
@@ -592,7 +663,7 @@ int mutt_save_message (HEADER *h, int delete, int decode, int decrypt, int *redr
 
     need_buffy_cleanup = (ctx.magic == M_MBOX || ctx.magic == M_MMDF || ctx.magic == M_KENDRA);
 
-    mx_close_mailbox (&ctx);
+    mx_close_mailbox (&ctx, NULL);
 
     if (need_buffy_cleanup)
     {
@@ -614,18 +685,23 @@ int mutt_save_message (HEADER *h, int delete, int decode, int decrypt, int *redr
     }
 
     mutt_clear_error ();
+    return (0);
   }
-
-  return (0);
+  
+  return -1;
 }
 
+/* XXX - merge this with mutt_pipe_message_to_state? */
+
 static void print_msg (FILE *fp, CONTEXT *ctx, HEADER *h)
 {
-  
+  int cmflags = 0;
+  int chflags = CH_FROM;
 
+  pipe_print_set_flags (option (OPTPRINTDECODE), &cmflags, &chflags);
 
-#ifdef _PGPPATH
-  if (h->pgp & PGPENCRYPT)
+#ifdef HAVE_PGP
+  if (option (OPTPRINTDECODE) && (h->pgp & PGPENCRYPT))
   {
     if (!pgp_valid_passphrase ())
       return;
@@ -633,10 +709,10 @@ static void print_msg (FILE *fp, CONTEXT *ctx, HEADER *h)
   }
 #endif
 
+  if (option (OPTPRINTDECODE))
+    mutt_parse_mime_message (ctx, h);
 
-
-  mutt_parse_mime_message (ctx, h);
-  mutt_copy_message (fp, ctx, h, M_CM_DECODE, CH_WEED | CH_DECODE | CH_REORDER);
+  mutt_copy_message (fp, ctx, h, cmflags, chflags);
 }
 
 void mutt_print_message (HEADER *h)
@@ -680,3 +756,55 @@ void mutt_version (void)
 {
   mutt_message ("Mutt %s (%s)", MUTT_VERSION, ReleaseDate);
 }
+
+void mutt_edit_content_type (HEADER *h, BODY *b)
+{
+  char buf[LONG_STRING];
+  char tmp[STRING];
+  PARAMETER *p;
+  
+  snprintf (buf, sizeof (buf), "%s/%s", TYPE (b), b->subtype);
+  if (b->parameter)
+  {
+    size_t l;
+    
+    for (p = b->parameter; p; p = p->next)
+    {
+      l = strlen (buf);
+
+      rfc822_cat (tmp, sizeof (tmp), p->value, MimeSpecials);
+      snprintf (buf + l, sizeof (buf) - l, "; %s=%s", p->attribute, tmp);
+    }
+  }
+  
+  if (mutt_get_field ("Content-Type: ", buf, sizeof (buf), 0) != 0 ||
+      buf[0] == 0)
+    return;
+  
+  /* clean up previous junk */
+  mutt_free_parameter (&b->parameter);
+  FREE (&b->subtype);
+  
+  mutt_parse_content_type (buf, b);
+  
+  mutt_message ("Content-Type changed to %s/%s.", TYPE (b),
+		NONULL (b->subtype));
+  
+  if (!is_multipart (b) && b->parts)
+    mutt_free_body (&b->parts);
+  if (!mutt_is_message_type (b->type, b->subtype) && b->hdr)
+  {
+    b->hdr->content = NULL;
+    mutt_free_header (&b->hdr);
+  }
+
+#ifdef HAVE_PGP
+  if (h)
+  {
+    if (h->content == b)
+      h->pgp = 0;
+    h->pgp |= pgp_query (b);
+  }
+#endif /* HAVE_PGP */
+
+}
