@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 #include "config.h"
@@ -27,6 +27,7 @@
 #include <time.h>
 #include <limits.h>
 #include <stdarg.h>
+#include <signal.h>
 
 #ifndef _POSIX_PATH_MAX
 #include <posix1_lim.h>
@@ -35,19 +36,6 @@
 #include "rfc822.h"
 #include "hash.h"
 
-#ifdef ENABLE_NLS
-# include <libintl.h>
-# define _(a) (gettext (a))
-# ifdef gettext_noop
-#  define N_(a) gettext_noop (a)
-# else
-#  define N_(a) (a)
-# endif
-#else
-# define _(a) (a)
-# define N_(a) a
-#endif
-
 #ifdef SUBVERSION
 # define MUTT_VERSION (VERSION SUBVERSION)
 #else  
@@ -63,14 +51,6 @@
 #define INITVAL(x) 
 #endif
 
-#define TRUE 1
-#define FALSE 0
-
-#define HUGE_STRING	5120
-#define LONG_STRING     1024
-#define STRING          256
-#define SHORT_STRING    128
-
 /* flags for mutt_copy_header() */
 #define CH_UPDATE	1      /* update the status and x-status fields? */
 #define CH_WEED		(1<<1) /* weed the headers? */
@@ -85,6 +65,8 @@
 #define CH_UPDATE_LEN	(1<<10) /* update Lines: and Content-Length: */
 #define CH_TXTPLAIN	(1<<11) /* generate text/plain MIME headers */
 #define CH_NOLEN	(1<<12) /* don't write Content-Length: and Lines: */
+#define CH_WEED_DELIVERED (1<<13) /* weed eventual Delivered-To headers */
+#define CH_FORCE_FROM	(1<<14)	/* give CH_FROM precedence over CH_WEED? */
 
 /* flags for mutt_enter_string() */
 #define  M_ALIAS   1      /* do alias "completion" by calling up the alias-menu */
@@ -140,8 +122,9 @@ typedef enum
 #define M_SENDHOOK	(1<<2)
 #define M_FCCHOOK	(1<<3)
 #define M_SAVEHOOK	(1<<4)
-#ifdef _PGPPATH
-#define M_PGPHOOK	(1<<5)
+#define M_CHARSETHOOK	(1<<5)
+#ifdef HAVE_PGP
+#define M_PGPHOOK	(1<<6)
 #endif
 
 /* tree characters for linearize_tree and print_enriched_string */
@@ -211,7 +194,7 @@ enum
   M_PERSONAL_RECIP,
   M_PERSONAL_FROM,
   M_ADDRESS,
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
   M_PGP_SIGN,
   M_PGP_ENCRYPT,
   M_PGP_KEY,
@@ -225,9 +208,13 @@ enum
 
   /* options for socket code */
   M_NEW_SOCKET,
+#ifdef USE_SSL
+  M_NEW_SSL_SOCKET,
+#endif
 
   /* Options for mutt_save_attachment */
-  M_SAVE_APPEND
+  M_SAVE_APPEND,
+  M_SAVE_OVERWRITE
 };
 
 /* possible arguments to set_quadoption() */
@@ -243,13 +230,15 @@ enum
 enum
 {
 
-#ifdef _PGPPATH
-  OPT_VERIFYSIG, /* verify PGP signatures */
+#ifdef HAVE_PGP
+  OPT_VERIFYSIG,      /* verify PGP signatures */
+  OPT_PGPTRADITIONAL, /* create old-style PGP messages */
 #endif
 
   OPT_PRINT,
   OPT_INCLUDE,
   OPT_DELETE,
+  OPT_MFUPTO,
   OPT_MIMEFWD,
   OPT_MOVE,
   OPT_COPY,
@@ -258,7 +247,9 @@ enum
   OPT_REPLYTO,
   OPT_ABORT,
   OPT_RECALL,
-  OPT_SUBJECT
+  OPT_SUBJECT,
+  OPT_MIMEFWDREST,
+  OPT_MAX
 };
 
 /* flags to ci_send_message() */
@@ -270,7 +261,7 @@ enum
 #define SENDBATCH	(1<<5)
 #define SENDMAILX	(1<<6)
 #define SENDKEY		(1<<7)
-#define SENDEDITMSG	(1<<8)
+#define SENDRESEND	(1<<8)
 
 /* boolean vars */
 enum
@@ -285,23 +276,41 @@ enum
   OPTAUTOTAG,
   OPTBEEP,
   OPTBEEPNEW,
+  OPTBOUNCEDELIVERED,
   OPTCHECKNEW,
   OPTCOLLAPSEUNREAD,
   OPTCONFIRMAPPEND,
   OPTCONFIRMCREATE,
+  OPTDELETEUNTAG,
   OPTEDITHDRS,
+  OPTENCODEFROM,
+  OPTENVFROM,
   OPTFASTREPLY,
   OPTFCCATTACH,
+  OPTFCCCLEAR,
   OPTFOLLOWUPTO,
   OPTFORCENAME,
   OPTFORWDECODE,
-  OPTFORWWEEDHEADER,
   OPTFORWQUOTE,
   OPTHDRS,
   OPTHEADER,
   OPTHELP,
   OPTHIDDENHOST,
   OPTIGNORELISTREPLYTO,
+#ifdef USE_IMAP
+  OPTIMAPLSUB,
+  OPTIMAPPASSIVE,
+  OPTIMAPSERVERNOISE,
+# ifdef USE_SSL
+  OPTIMAPFORCESSL,
+# endif
+#endif
+#ifdef USE_SSL
+  OPTSSLV2,
+  OPTSSLV3,
+  OPTTLSV1,
+  OPTSSLSYSTEMCERTS,
+#endif
   OPTIMPLICITAUTOVIEW,
   OPTMAILCAPSANITIZE,
   OPTMARKERS,
@@ -316,15 +325,19 @@ enum
   OPTPIPESPLIT,
   OPTPOPDELETE,
   OPTPOPLAST,
+  OPTPRINTDECODE,
   OPTPROMPTAFTER,
   OPTREADONLY,
   OPTREPLYSELF,
   OPTRESOLVE,
   OPTREVALIAS,
   OPTREVNAME,
+  OPTREVREAL,
+  OPTRFC2047PARAMS,
   OPTSAVEADDRESS,
   OPTSAVEEMPTY,
   OPTSAVENAME,
+  OPTSCORE,
   OPTSIGDASHES,
   OPTSORTRE,
   OPTSTATUSONTOP,
@@ -341,18 +354,24 @@ enum
   OPTWRAP,
   OPTWRAPSEARCH,
   OPTWRITEBCC,		/* write out a bcc header? */
+  OPTXMAILER,
 
   /* PGP options */
   
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
   OPTPGPAUTOSIGN,
   OPTPGPAUTOENCRYPT,
   OPTPGPLONGIDS,
   OPTPGPREPLYENCRYPT,
   OPTPGPREPLYSIGN,
+  OPTPGPREPLYSIGNENCRYPTED,
+#if 0
   OPTPGPENCRYPTSELF,
+#endif
+  OPTPGPRETAINABLESIG,
   OPTPGPSTRICTENC,
   OPTFORWDECRYPT,
+  OPTPGPSHOWUNUSABLE,
 #endif
 
   /* pseudo options */
@@ -366,16 +385,20 @@ enum
   OPTMSGERR,		/* (pseudo) used by mutt_error/mutt_message */
   OPTSEARCHINVALID,	/* (pseudo) used to invalidate the search pat */
   OPTSIGNALSBLOCKED,	/* (pseudo) using by mutt_block_signals () */
+  OPTSYSSIGNALSBLOCKED,	/* (pseudo) using by mutt_block_signals_system () */
   OPTNEEDRESORT,	/* (pseudo) used to force a re-sort */
   OPTVIEWATTACH,	/* (pseudo) signals that we are viewing attachments */
   OPTFORCEREDRAWINDEX,	/* (pseudo) used to force a redraw in the main index */
   OPTFORCEREDRAWPAGER,	/* (pseudo) used to force a redraw in the pager */
   OPTSORTSUBTHREADS,	/* (pseudo) used when $sort_aux changes */
   OPTNEEDRESCORE,	/* (pseudo) set when the `score' command is used */
-  OPTUSEHEADERDATE,	/* (pseudo) used by edit-message */
   OPTATTACHMSG,		/* (pseudo) used by attach-message */
+  OPTKEEPQUIET,		/* (pseudo) shut up the message and refresh
+			 * 	    functions while we are executing an
+			 * 	    external program.
+			 */
   
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
   OPTPGPCHECKTRUST,	/* (pseudo) used by pgp_select_key () */
   OPTDONTHANDLEPGPKEYS,	/* (pseudo) used to extract PGP keys */
 #endif
@@ -397,10 +420,9 @@ enum
 #define toggle_option(x) mutt_bit_toggle(Options,x)
 #define option(x) mutt_bit_isset(Options,x)
 
-/* Bit fields for ``Signals'' */
-#define S_INTERRUPT (1<<1)
-#define S_SIGWINCH  (1<<2)
-#define S_ALARM     (1<<3)
+/* Exit values used in send_msg() */
+#define S_ERR 127
+#define S_BKG 126
 
 typedef struct list_t
 {
@@ -409,7 +431,6 @@ typedef struct list_t
 } LIST;
 
 #define mutt_new_list() safe_calloc (1, sizeof (LIST))
-void mutt_add_to_list (LIST **, BUFFER *);
 void mutt_free_list (LIST **);
 int mutt_matches_ignore (const char *, LIST *);
 
@@ -464,7 +485,7 @@ typedef struct content
   unsigned int binary : 1; /* long lines, or CR not in CRLF pair */
   unsigned int from : 1;   /* has a line beginning with "From "? */
   unsigned int dot : 1;    /* has a line consisting of a single dot? */
-  unsigned int nonasc : 1; /* has unicode characters out of ASCII range */
+  unsigned int cr : 1;     /* has CR, even when in a CRLF pair */
 } CONTENT;
 
 typedef struct body
@@ -502,23 +523,28 @@ typedef struct body
 				 * encoding update.
 				 */
   
-  unsigned int type : 3;        /* content-type primary type */
+  unsigned int type : 4;        /* content-type primary type */
   unsigned int encoding : 3;    /* content-transfer-encoding */
   unsigned int disposition : 2; /* content-disposition */
-  unsigned int use_disp : 1;    /* Content-Disposition field printed? */
+  unsigned int use_disp : 1;    /* Content-Disposition uses filename= ? */
   unsigned int unlink : 1;      /* flag to indicate the the file named by
 				 * "filename" should be unlink()ed before
 				 * free()ing this structure
 				 */
   unsigned int tagged : 1;
   unsigned int deleted : 1;	/* attachment marked for deletion */
+  unsigned int noconv : 1;	/* don't do character set conversion */
+
+#ifdef HAVE_PGP
+  unsigned int goodsig : 1;	/* good PGP signature */
+#endif
 
 } BODY;
 
 typedef struct header
 {
-#ifdef _PGPPATH
-  unsigned int pgp : 3;
+#ifdef HAVE_PGP
+  unsigned int pgp : 4;
 #endif
 
   unsigned int mime : 1;    /* has a Mime-Version header? */
@@ -577,6 +603,13 @@ typedef struct header
   struct header *last_sort; /* last message in subthread, for secondary SORT_LAST */
   char *tree;            /* character string to print thread tree */
 
+#ifdef MIXMASTER
+  LIST *chain;
+#endif
+
+#ifdef USE_IMAP
+  void *data;            /* driver-specific data (only used by IMAP */
+#endif
 } HEADER;
 
 #include "mutt_regex.h"
@@ -635,9 +668,7 @@ typedef struct
   unsigned int readonly : 1;    /* don't allow changes to the mailbox */
   unsigned int dontwrite : 1;   /* dont write the mailbox on close */
   unsigned int append : 1;	/* mailbox is opened in append mode */
-  unsigned int setgid : 1;
   unsigned int quiet : 1;	/* inhibit status messages? */
-  unsigned int revsort : 1;	/* mailbox sorted in reverse? */
   unsigned int collapsed : 1;   /* are all threads collapsed? */
   unsigned int closing : 1;	/* mailbox is being closed */
 } CONTEXT;
@@ -664,7 +695,7 @@ typedef struct
 
 
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
 #define M_VERIFY	(1<<1) /* perform signature verification */
 #endif
 
@@ -672,6 +703,7 @@ typedef struct
 
 #define M_PENDINGPREFIX (1<<2) /* prefix to write, but character must follow */
 #define M_WEED          (1<<3) /* weed headers even when not in display mode */
+#define M_CHARCONV	(1<<4) /* Do character set conversions */
 
 #define state_set_prefix(s) ((s)->flags |= M_PENDINGPREFIX)
 #define state_reset_prefix(s) ((s)->flags &= ~M_PENDINGPREFIX)
@@ -682,4 +714,5 @@ void state_prefix_putc(char, STATE *);
 int  state_printf(STATE *, const char *, ...);
 
 #include "protos.h"
+#include "lib.h"
 #include "globals.h"
