@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 /*
@@ -63,7 +63,8 @@ static int getPass (void)
   {
     char tmp[SHORT_STRING];
     tmp[0] = '\0';
-    if (mutt_get_password ("POP Password: ", tmp, sizeof (tmp)) != 0)
+    if (mutt_get_password (_("POP Password: "), tmp, sizeof (tmp)) != 0
+	|| *tmp == '\0')
       return 0;
     PopPass = safe_strdup (tmp);
   }
@@ -81,19 +82,19 @@ void mutt_fetchPopMail (void)
   struct hostent *he;
   char buffer[2048];
   char msgbuf[SHORT_STRING];
-  int s, i, msgs, bytes, err = 0;
+  int s, i, last = 0, msgs, bytes, err = 0;
   CONTEXT ctx;
   MESSAGE *msg = NULL;
 
   if (!PopHost)
   {
-    mutt_error ("POP host is not defined.");
+    mutt_error _("POP host is not defined.");
     return;
   }
 
   if (!PopUser)
   {
-    mutt_error ("No POP username is defined.");
+    mutt_error _("No POP username is defined.");
     return;
   }
     
@@ -105,12 +106,12 @@ void mutt_fetchPopMail (void)
   sin.sin_family = AF_INET;
   sin.sin_port = htons (PopPort);
 
-  if ((n = inet_addr (PopHost)) == -1)
+  if ((n = inet_addr (NONULL(PopHost))) == -1)
   {
     /* Must be a DNS name */
-    if ((he = gethostbyname (PopHost)) == NULL)
+    if ((he = gethostbyname (NONULL(PopHost))) == NULL)
     {
-      mutt_error ("Could not find address for host %s.", PopHost);
+      mutt_error (_("Could not find address for host %s."), PopHost);
       return;
     }
     memcpy ((void *)&sin.sin_addr, *(he->h_addr_list), he->h_length);
@@ -118,7 +119,7 @@ void mutt_fetchPopMail (void)
   else
     memcpy ((void *)&sin.sin_addr, (void *)&n, sizeof(n));
   
-  mutt_message ("Connecting to %s", inet_ntoa (sin.sin_addr));
+  mutt_message (_("Connecting to %s"), inet_ntoa (sin.sin_addr));
 
   if (connect (s, (struct sockaddr *) &sin, sizeof (struct sockaddr_in)) == -1)
   {
@@ -129,37 +130,40 @@ void mutt_fetchPopMail (void)
   if (getLine (s, buffer, sizeof (buffer)) == -1)
     goto fail;
 
-  if (strncmp (buffer, "+OK", 3) != 0)
+  if (mutt_strncmp (buffer, "+OK", 3) != 0)
   {
     mutt_remove_trailing_ws (buffer);
-    mutt_error (buffer);
+    mutt_error ("%s", buffer);
     goto finish;
   }
 
-  sprintf (buffer, "user %s\r\n", PopUser);
-  write (s, buffer, strlen (buffer));
+  snprintf (buffer, sizeof(buffer), "user %s\r\n", PopUser);
+  write (s, buffer, mutt_strlen (buffer));
 
   if (getLine (s, buffer, sizeof (buffer)) == -1)
     goto fail;
 
-  if (strncmp (buffer, "+OK", 3) != 0)
+  if (mutt_strncmp (buffer, "+OK", 3) != 0)
   {
     mutt_remove_trailing_ws (buffer);
-    mutt_error (buffer);
+    mutt_error ("%s", buffer);
     goto finish;
   }
   
-  sprintf (buffer, "pass %s\r\n", PopPass);
-  write (s, buffer, strlen (buffer));
+  snprintf (buffer, sizeof(buffer), "pass %s\r\n", NONULL(PopPass));
+  write (s, buffer, mutt_strlen (buffer));
   
   if (getLine (s, buffer, sizeof (buffer)) == -1)
     goto fail;
 
-  if (strncmp (buffer, "+OK", 3) != 0)
+  if (mutt_strncmp (buffer, "+OK", 3) != 0)
   {
-    PopPass[0] = 0; /* void the given password */
+    if(PopPass)
+      memset(PopPass, 0, mutt_strlen(PopPass));
+    
+    safe_free((void **) &PopPass); /* void the given password */
     mutt_remove_trailing_ws (buffer);
-    mutt_error (buffer[0] ? buffer : "Server closed connection!");
+    mutt_error ("%s", buffer[0] ? buffer : _("Server closed connection!"));
     goto finish;
   }
   
@@ -169,10 +173,10 @@ void mutt_fetchPopMail (void)
   if (getLine (s, buffer, sizeof (buffer)) == -1)
     goto fail;
 
-  if (strncmp (buffer, "+OK", 3) != 0)
+  if (mutt_strncmp (buffer, "+OK", 3) != 0)
   {
     mutt_remove_trailing_ws (buffer);
-    mutt_error (buffer);
+    mutt_error ("%s", buffer);
     goto finish;
   }
   
@@ -180,21 +184,36 @@ void mutt_fetchPopMail (void)
 
   if (msgs == 0)
   {
-    mutt_message ("No new mail in POP mailbox.");
+    mutt_message _("No new mail in POP mailbox.");
     goto finish;
   }
 
-  if (mx_open_mailbox (Spoolfile, M_APPEND, &ctx) == NULL)
+  if (mx_open_mailbox (NONULL(Spoolfile), M_APPEND, &ctx) == NULL)
     goto finish;
 
+  /* only get unread messages */
+  if(option(OPTPOPLAST))
+  {
+    write (s, "last\r\n", 6);
+    if (getLine (s, buffer, sizeof (buffer)) == -1)
+      goto fail;
+    
+    if (mutt_strncmp (buffer, "+OK", 3) == 0)
+      sscanf (buffer, "+OK %d", &last);
+    else
+      /* ignore an error here and assume all messages are new */
+      last = 0;
+  }
+  
   snprintf (msgbuf, sizeof (msgbuf),
-	    "Reading %d new message%s (%d bytes)...", msgs, msgs > 1 ? "s" : "", bytes);
+	    msgs > 1 ? _("Reading new messages (%d bytes)...") :
+		    _("Reading new message (%d bytes)..."), bytes);
   mutt_message (msgbuf);
 
-  for (i = 1 ; i <= msgs ; i++)
+  for (i = last + 1 ; i <= msgs ; i++)
   {
-    sprintf (buffer, "retr %d\r\n", i);
-    write (s, buffer, strlen (buffer));
+    snprintf (buffer, sizeof(buffer), "retr %d\r\n", i);
+    write (s, buffer, mutt_strlen (buffer));
 
     if (getLine (s, buffer, sizeof (buffer)) == -1)
     {
@@ -202,10 +221,10 @@ void mutt_fetchPopMail (void)
       goto fail;
     }
 
-    if (strncmp (buffer, "+OK", 3) != 0)
+    if (mutt_strncmp (buffer, "+OK", 3) != 0)
     {
       mutt_remove_trailing_ws (buffer);
-      mutt_error (buffer);
+      mutt_error ("%s", buffer);
       break;
     }
 
@@ -223,7 +242,7 @@ void mutt_fetchPopMail (void)
 
       if ((chunk = getLine (s, buffer, sizeof (buffer))) == -1)
       {
-	mutt_error ("Error reading message!");
+	mutt_error _("Error reading message!");
 	err = 1;
 	break;
       }
@@ -231,7 +250,7 @@ void mutt_fetchPopMail (void)
       /* check to see if we got a full line */
       if (buffer[chunk-2] == '\r' && buffer[chunk-1] == '\n')
       {
-	if (strcmp(".\r\n", buffer) == 0)
+	if (mutt_strcmp(".\r\n", buffer) == 0)
 	{
 	  /* end of message */
 	  break;
@@ -257,38 +276,48 @@ void mutt_fetchPopMail (void)
       fwrite (p, 1, chunk, msg->fp);
     }
 
-    if (mx_close_message (&msg) != 0)
+    if (mx_commit_message (msg, &ctx) != 0)
     {
-      mutt_error ("Error while writing mailbox!");
+      mutt_error _("Error while writing mailbox!");
       err = 1;
     }
 
+    mx_close_message (&msg);
+
     if (err)
       break;
 
     if (option (OPTPOPDELETE))
     {
       /* delete the message on the server */
-      sprintf (buffer, "dele %d\r\n", i);
-      write (s, buffer, strlen (buffer));
+      snprintf (buffer, sizeof(buffer), "dele %d\r\n", i);
+      write (s, buffer, mutt_strlen (buffer));
 
       /* eat the server response */
       getLine (s, buffer, sizeof (buffer));
-      if (strncmp (buffer, "+OK", 3) != 0)
+      if (mutt_strncmp (buffer, "+OK", 3) != 0)
       {
 	err = 1;
         mutt_remove_trailing_ws (buffer);
-	mutt_error (buffer);
+	mutt_error ("%s", buffer);
 	break;
       }
     }
+    
+    if ( msgs > 1)
+      mutt_message (_("%s [%d of %d messages read]"), msgbuf, i, msgs);
+    else
+      mutt_message (_("%s [%d message read]"), msgbuf, msgs);
 
-    mutt_message ("%s [%d messages read]", msgbuf, i);
   }
 
   if (msg)
+  {
+    if (mx_commit_message (msg, &ctx) != 0)
+      err = 1;
     mx_close_message (&msg);
-  mx_close_mailbox (&ctx);
+  }
+  mx_close_mailbox (&ctx, NULL);
 
   if (err)
   {
@@ -309,6 +338,6 @@ finish:
 
 fail:
 
-  mutt_error ("Server closed connection!");
+  mutt_error _("Server closed connection!");
   close (s);
 }
